#-- encoding: UTF-8

#-- copyright
# OpenProject is a project management system.
# Copyright (C) 2012-2018 the OpenProject Foundation (OPF)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License version 3.
#
# OpenProject is a fork of ChiliProject, which is a fork of Redmine. The copyright follows:
# Copyright (C) 2006-2017 Jean-Philippe Lang
# Copyright (C) 2010-2013 the ChiliProject Team
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# See docs/COPYRIGHT.rdoc for more details.
#++

class AddHierarchyPaths < ActiveRecord::Migration[5.1]
  def change
    create_table :hierarchy_paths, id: :integer do |t|
      t.belongs_to :work_package, index: { unique: true }
      # (255 * 3) = ca 767 bytes is the max length for an index in mysql 5.6 InnoDB
      t.string :path, null: false, limit: 255

      t.index :path
    end

    reversible do |dir|
      dir.up do
        Relation.rebuild_hierarchy_paths!
      rescue StandardError => e
        warn "Failed to rebuild hierarchy paths. Call `Relation.rebuild_hierarchy_paths!` manually to correct this: #{e}"
      end
    end
  end
end
